/*
 * Copyright (C) 2006 Jonathan McDowell <noodles@earth.li>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "delay.h"
#include "types.h"
#include "uart.h"

#define READ_TIMER (*(volatile unsigned long *)(0xFFFEC508))
#define TIMER_LOAD_VAL 0xffffffff

#define CFG_HZ 46875

static unsigned long timestamp;
static unsigned long lastdec;

unsigned long get_timer_masked(void)
{
	unsigned long now = READ_TIMER;

	if (lastdec >= now) {
		timestamp += lastdec - now;
	} else {
		timestamp += lastdec + TIMER_LOAD_VAL - now;
	}
	lastdec = now;

	return timestamp;
}

void reset_timer_masked(void)
{
	lastdec = READ_TIMER;
	timestamp = 0;
}

void udelay(unsigned long usecs)
{
	unsigned long tmo, tmp;

	if (usecs >= 1000) {
		tmo = usecs >> 10;
		tmo *= CFG_HZ;
		tmo = tmo >> 10;
	} else {
		tmo = usecs * CFG_HZ;
		tmo = tmo >> 20;	/* Not quite 1000, but close enough */
	}

	tmp = get_timer_masked();
	if ((tmo + tmp + 1) < tmp)
		reset_timer_masked();
	else
		tmo += tmp;

	while (get_timer_masked() < tmo)
		;
}

void delay_init(void)
{
	u32 val;

	*((u32 *) (0xFFFEC504)) = TIMER_LOAD_VAL;
	val = (1 << 0) | (1 << 1) | (1 << 5) | (7 << 2);
	*((u32 *) (0xFFFEC500)) = val;

	reset_timer_masked();
}
