/*
 * Copyright (C) 2006 Jonathan McDowell <noodles@earth.li>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "types.h"
#include "uart.h"

#define FCR_FIFO_EN	0x01		/* FIFO enable */
#define FCR_RXSR	0x02		/* Receiver soft reset */
#define FCR_TXSR	0x04		/* Transmitter soft reset */

#define LCR_BKSE	0x80		/* Bank select enable */

#define LSR_DR		0x01		/* Data ready */
#define LSR_THRE	0x20		/* Xmit holding register empty */

#define	MCR_DTR		0x01
#define	MCR_RTS		0x02

#define uart_rbr	(*(volatile u8 *) 0xFFFB0000)
#define uart_ier	(*(volatile u8 *) 0xFFFB0004)
#define uart_fcr	(*(volatile u8 *) 0xFFFB0008)
#define uart_lcr	(*(volatile u8 *) 0xFFFB000C)
#define uart_mcr	(*(volatile u8 *) 0xFFFB0010)
#define uart_lsr	(*(volatile u8 *) 0xFFFB0014)
#define uart_msr	(*(volatile u8 *) 0xFFFB0018)
#define uart_scr	(*(volatile u8 *) 0xFFFB001C)
#define uart_mdr1	(*(volatile u8 *) 0xFFFB0020)
#define uart_12m_sel	(*(volatile u8 *) 0xFFFB004C)

void uart_putc(char c)
{
	while ((uart_lsr & LSR_THRE) == 0) ;
	uart_rbr = c;
}

void uart_puts(char *s)
{
	int i;

	for (i = 0; s[i] != 0; i++)
		uart_putc(s[i]);
}

void uart_putnibble(u8 b)
{
	b &= 0xf;
	if (b < 10)
		b += '0';
	else
		b += 'A' - 10;

	uart_putc(b);
}	

void uart_putbyte(u8 b)
{
	uart_putnibble(b >> 4);
	uart_putnibble(b & 0xf);
}

void uart_putword(u32 b)
{
	uart_putbyte(b >> 24);
	uart_putbyte((b >> 16) & 0xff);
	uart_putbyte((b >>  8) & 0xff);
	uart_putbyte(b & 0xff);
}

char uart_getc(void)
{
	while ((uart_lsr & LSR_DR) == 0) ;

	return uart_rbr;
}

void uart_init(void)
{
	uart_12m_sel = 1;
	uart_ier = 0;
	uart_mdr1 = 0x7;
	uart_lcr = LCR_BKSE | 0x3;
	uart_rbr = 1;				/* dll */
	uart_ier = 0;				/* dlh */
	uart_lcr = 0x03;			/* 8N1 */
	uart_mcr = MCR_DTR | MCR_RTS;
	uart_fcr = FCR_FIFO_EN | FCR_RXSR | FCR_TXSR;
	uart_mdr1 = 0;
}
